@@ -4,8 +4,11 @@
 
 use App::Music::ChordPro::Output::Common;
 use App::Music::ChordPro::Output::PDF;
+use App::Music::ChordPro::Output::PDF::PDFWriter;
+use App::Music::ChordPro::Output::PDF::StringDiagrams;
 use App::Music::ChordPro::Output::Debug;
 use App::Music::ChordPro::Output::ChordPro;
 use App::Music::ChordPro::Output::Text;
+use App::Music::ChordPro::Output::HTML;
 
 1;
